//
//  RealmProtocol.swift
//  WherePrayer
//
//  Created by Jeff Shulman on 5/4/18.
//  Copyright © 2018 Lighthouse Woods LLC. All rights reserved.
//
//  Protocol for Realm. Needed in order to substitute mock version for unit testing

import Foundation
import Realm
import RealmSwift

// Remove "class" for 4.x version
// Add "class" for 3.x version

public protocol RealmProtocol {
    var isInWriteTransaction: Bool { get }
    
    init(configuration: Realm.Configuration) throws
    
    func write(withoutNotifying tokens: [NotificationToken], _ block: (() throws -> Void)) throws
    func beginWrite()
    func commitWrite(withoutNotifying tokens: [NotificationToken]) throws
    func cancelWrite()
    
    func add(_ object: RealmSwift.Object, update: RealmSwift.Realm.UpdatePolicy)
    func add<S>(_ objects: S, update: RealmSwift.Realm.UpdatePolicy) where S : Sequence, S.Element : RealmSwift.Object

    func create<T>(_ type: T.Type, value: Any, update: RealmSwift.Realm.UpdatePolicy) -> T  where T : RealmSwift.Object

    func delete(_ object: Object)
    func delete<S>(_ objects: S) where S : Sequence, S.Element : RealmSwift.Object
    func delete<Element>(_ objects: List<Element>) where Element : RealmSwift.Object
    func delete<Element>(_ objects: Results<Element>) where Element : RealmSwift.Object
    func deleteAll()

    func objects<Element: Object>(_ type: Element.Type) -> Results<Element>
    func object<Element: Object, KeyType>(ofType type: Element.Type, forPrimaryKey key: KeyType) -> Element?

    func observe(_ block: @escaping NotificationBlock) -> NotificationToken
}

extension Realm: RealmProtocol {}
