//
//  MockRealm.swift
//  WherePrayer
//
//  Created by Jeff Shulman on 5/4/18.
//  Copyright © 2018 Lighthouse Woods LLC. All rights reserved.
//

import RealmSwift
import RealmBug

final class MockRealm: RealmProtocol {
    var isInWriteTransaction: Bool {
        get {
            return passThru.isInWriteTransaction
        }
    }
    
    private var passThru: Realm
    
    static func cleanupMocks() {
        MockRealm.sThrowInitError = false
        MockRealm.sThrowWriteError = false
        MockRealm.sThrowCommitWriteError = false
    }
    
    static var sThrowInitError: Bool = false
    init(configuration: Realm.Configuration) throws {
        if MockRealm.sThrowInitError {
            // Throw any error
            throw LocalDataLayerError.noSelfReference("\(#function) \(#line)")
        }
        
        passThru = try Realm(configuration: configuration)
    }
    
    deinit {
        MockRealm.cleanupMocks()
    }
    
    static var sThrowWriteError: Bool = false
    func write(withoutNotifying tokens: [NotificationToken], _ block: (() throws -> Void)) throws {
        if MockRealm.sThrowWriteError {
            // Throw any error
            throw LocalDataLayerError.noSelfReference("\(#function) \(#line)")
        }
        
        try passThru.write(withoutNotifying: tokens, block)
    }
    
    func beginWrite() {
        passThru.beginWrite()
    }
    
    static var sThrowCommitWriteError: Bool = false
    func commitWrite(withoutNotifying tokens: [NotificationToken]) throws {
        if MockRealm.sThrowCommitWriteError {
            // Throw any error
            throw LocalDataLayerError.noSelfReference("\(#function) \(#line)")
        }
        
        try passThru.commitWrite(withoutNotifying: tokens)
//        WPLog.log.debug("commitWrite on \(passThru)")
    }
    
    func cancelWrite() {
        passThru.cancelWrite()
    }
    
    func add(_ object: Object, update: RealmSwift.Realm.UpdatePolicy) {
        passThru.add(object, update: update)
    }
    
    func add<S>(_ objects: S, update: RealmSwift.Realm.UpdatePolicy) where S : Sequence, S.Element : RealmSwift.Object {
        passThru.add(objects, update: update)
    }
    
    func create<T>(_ type: T.Type, value: Any, update: RealmSwift.Realm.UpdatePolicy) -> T  where T : RealmSwift.Object {
        return passThru.create(type, value: value, update: update)
    }
    
    func delete(_ object: Object) {
        passThru.delete(object)
    }
    
    func delete<S>(_ objects: S) where S : Sequence, S.Element : RealmSwift.Object {
        passThru.delete(objects)
    }
    
    func delete<Element>(_ objects: List<Element>) where Element : RealmSwift.Object {
        passThru.delete(objects)
    }
    
    func delete<Element>(_ objects: Results<Element>) where Element : RealmSwift.Object {
        passThru.delete(objects)
    }
    
    func deleteAll() {
        passThru.deleteAll()
    }
    
    func objects<Element>(_ type: Element.Type) -> Results<Element> where Element : Object {
        let results = passThru.objects(type)

//        WPLog.log.debug("objects on \(passThru) \(results)")

        return results
    }
    
    func object<Element, KeyType>(ofType type: Element.Type, forPrimaryKey key: KeyType) -> Element? where Element : Object {
        let result = passThru.object(ofType: type, forPrimaryKey: key)
        
//        WPLog.log.debug("object on \(passThru) \(String(describing: result))")

        return result
    }
    
    func observe(_ block: @escaping NotificationBlock) -> NotificationToken {
        return passThru.observe(block)
    }
}
