//
//  RealmBugTests.swift
//  RealmBugTests
//
//  Created by Jeff Shulman on 11/28/19.
//  Copyright © 2019 Jeff Shulman. All rights reserved.
//

import XCTest
import MapKit
import Nimble
import RealmSwift
import Swinject
@testable import RealmBug

struct MapData {
    struct Locations {
        static let empireStateBuilding = CLLocationCoordinate2DMake(40.7484, -73.9857)
    }
}

class RealmBugTests: XCTestCase {
    var globalRegistry: DependencyRegistryModel!
    var container: Container!
    
    override func setUp() {
        super.setUp()
        
        container = Container()
        globalRegistry = MockDependencyRegistryModel(container: container)
    }

    override func tearDown() {
        container.removeAll()
        globalRegistry = nil
        
        super.tearDown()
    }

    func testAboutToDeleteCascadeImages() {
        let dataLayer = globalRegistry.makeLocalDataLayer() as! MockLocalDataLayer
        let realm = try! dataLayer.realm()
        
        let homeAddress = AddressObject(coordinates: MapData.Locations.empireStateBuilding, name: "Empire State")
        expect(homeAddress.aboutToDeleteCascadeImages(in: realm)).to(beNil())
        
        let imageID = UUID().uuidString
        let image = ImageObject(id: imageID)
        homeAddress.mapImage = image
        
        expect {
            try realm.write(withoutNotifying: []) {
                realm.add(image, update: .error)
                realm.add(homeAddress, update: .error)
            }
        }.toNot(throwError())
        
        expect(ImageObject.unreferencedPredicate().evaluate(with: image)).to(beFalse())
        
//        // Delete address
//        expect {
//            try realm.write(withoutNotifying: []) {
//                let deletedImageID = homeAddress.aboutToDeleteCascadeImages(in: realm)
//                expect(deletedImageID).toNot(beNil())
//                expect(deletedImageID) == imageID
//                realm.delete(homeAddress)
//            }
//        }.toNot(throwError())
//        
//        expect(dataLayer.asyncOperationsCompleted()).toEventually(beTrue(), timeout: 5)
    }
}
